#ifndef _INCLUDED_FILE_RING_H
#define _INCLUDED_FILE_RING_H

#include <Foundation/NSObject.h>

enum FRPosition
{
    PosBeforeCur = 0,
    PosAfterCur,
    PosBegin,
    PosAtEnd
};

@class NSString;
@class NSArray;
@class NSMutableArray;

@interface FileRing : NSObject
{
    NSArray * _allowedTypes;
    NSMutableArray * _files;
    unsigned _pos;
    unsigned _writePos;
}

- (id) init;
- (id) initWithTypes: (NSArray *) allowedTypes;

// Inserts elements from 'paths' into the ring, expanding directories,
// current position  is set to 0.
- (id) initFromPathArray: (NSArray *)paths withTypes: (NSArray *) allowedTypes;

// Inserts elements from 'paths' into the ring, expanding directories,
// current position is set to first inserted or found element
- (void) insertPathArray: (NSArray *) paths positioning: (int) posMode;

// Remove the current content and insert the content of a directory
// contaning path. Current position is set to path.
- (void) replaceWithDirContainingPath: (NSString *) path;

// Inserts file based on the file type and withSibl flag.
//
//  Path          withSibl   Action
//  ----------    --------   -----------
//  regular file   YES       Insert all allowed files from path's directory
//  regular file   NO        Insert one file
//  directory      YES       Insert the content of directory
//  directory      NO        The same (for directories withSibl is ignored)
// 
// Returns the position of the first inserted or found element,
// or NSNotFound
- (void) insertPath: (NSString *) path withSiblings: (BOOL) withSibl
        positioning: (int) posMode;

- (void) removeAllObjects;

- (NSArray *) allowedTypes;
- (void) setAllowedTypes: (NSArray *) allowedTypes;

- (unsigned) count;

- (NSString *) currentPath;
- (NSString *) currentDir;
- (NSString *) currentName;
- (NSString *) nextPath;
- (NSString *) prevPath;


@end // @interface FileRing

#endif // _INCLUDED_FILE_RING_H
