#include "Fragment.h"
#include <AppKit/NSImage.h>
#include <Foundation/NSGeometry.h>
#include <Foundation/NSException.h>

@implementation Fragment


+ (id) fragmentWithScene: (NSRect) scene
{
    return AUTORELEASE([[Fragment alloc] initWithScene: scene]);
}

+ (id) fragmentWithScene: (NSRect) scene
                rotation: (int) degrees
{
    return AUTORELEASE([[Fragment alloc] initWithScene: scene
                                         rotation: degrees]);
}

+ (id) fragmentWithImage: (NSImage *) image
{
    return AUTORELEASE([[Fragment alloc] initWithImage: image]);
}

+ (id) fragmentWithImage: (NSImage *) image
                rotation: (int) degrees
{
    return AUTORELEASE([[Fragment alloc] initWithImage: image
                                         rotation: degrees]);
}

- (id) init
{
    return [self initWithScene: NSZeroRect rotation: 0];
}

- (id) initWithScene: (NSRect) scene
{
    return [self initWithScene: scene rotation: 0];
}

- initWithScene: (NSRect) scene
       rotation: (int) degrees
{
    _scene = scene;
    _rotation = degrees;
    return self;
}

- (id) initWithImage: (NSImage *) image
{
    return [self initWithImage: image rotation: 0];
}

- (id) initWithImage: (NSImage *) image
            rotation: (int) degrees
{
    return [self initWithScene:
                     NSMakeRect(0, 0, [image size].width, [image size].height)
                 rotation: degrees];
}

- (NSRect) scene
{
    return _scene;
}

- (void) setScene: (NSRect) scene
{
    _scene = scene;
}

- (int) span
{
    return MAX( _scene.size.width, _scene.size.height );
}

- (int) rotation
{
    return _rotation;
}

- (void) setRotation: (int) degrees
{
    _rotation = degrees;
}

- (float) scaleFor: (int) requiredSpan
{
    int span = MAX( _scene.size.width, _scene.size.height );
    if ( 0 < requiredSpan && requiredSpan < span )
        return requiredSpan/span;
    else
        return 1.0;
}

@end // @implementation Fragment
