#include "PHAffineTransform.h"
#include <Foundation/NSException.h>
#include "PHGeometry.h"

@implementation NSAffineTransform (PhotoClip)

+ (NSAffineTransform *) transformToQIofRect: (NSRect) rect
                                      scale: (float) scale
                                   rotation: (int) degrees
{
    NSAffineTransform * xform = [NSAffineTransform transform];
    [xform scaleBy: scale];
    [xform rotateByDegrees: degrees];

    NSRect transformed_rect = [xform transformRect: rect];


    // translate reference frame to map rectangle 'rect' into first quadrant
    [xform translateOrigCoordXBy: -transformed_rect.origin.x
                             yBy: -transformed_rect.origin.y];

    return xform;
}

- (NSRect) transformRect: (NSRect) rect
{
    NSPoint end = NSMakePoint(rect.origin.x + rect.size.width,
                              rect.origin.y + rect.size.height);
    NSPoint a = [self transformPoint: rect.origin];
    NSPoint b = [self transformPoint: end];

    return PHRectFromPoints(a, b);
}

- (void) translateOrigCoordXBy: (float) dx yBy: (float) dy
{
    NSAffineTransformStruct m = [self transformStruct];
    m.tX += dx;
    m.tY += dy;
    [self setTransformStruct: m];
}

@end // NSAffineTransform (PhotoClip)
