#ifndef _INCLUDED_PHGEOMETRY_H
#define _INCLUDED_PHGEOMETRY_H

#include <Foundation/NSGeometry.h>

@class NSAffineTransform;

NSPoint PHMovePoint(NSPoint pt, float dx, float dy);
NSPoint PHIntegralPoint(NSPoint pt);
NSRect PHRectFromPoints(NSPoint a, NSPoint b);
NSSize PHIntegerSize(NSSize s);
NSSize PHIntegerSizePreserveAR(NSSize s, float aspectRatio);
NSRect PHIntegerRect(NSRect rc);
NSRect PHIntegerRectPreserveAR(NSRect rc, float aspectRatio);

NSRect PHNearestIntegerRect(NSRect rc);
float PHDistance(NSPoint a, NSPoint b);
NSPoint PHClosestPointInRect(NSPoint pt, NSRect rc);
NSPoint PHClosestPointOnRectEdge(NSPoint pt, NSRect rc);
NSRect PHPushRectInsideBoundaries(NSRect aRect, NSRect bound);
NSRect PHStretchRectToSpan( NSRect rc, int requiredSpan, float requiredAR,
                            NSRect scene );

NSRect PHJMoveRectOrigin( NSRect rc, NSSize gridSize, NSSize imgSize  );
NSRect PHJMoveRectKeepAR( NSRect rc, NSSize gridSize, NSSize imgSize );
NSRect PHJMoveRectKeepSpan( NSRect rc, NSSize gridSize, NSSize imgSize  );

NSSize PHRotateSize(NSSize val, int degrees);
NSSize PHScaledSize( NSSize src, int newSpan, float * resultScale );
float PHMaxSpan( NSSize size );
NSAffineTransform * PHReverseTransform(NSAffineTransform * xform);

typedef struct _Vect2D
{
    float dx;
    float dy;
} Vect2D;

static inline Vect2D PHMakeVect2D(float dx, float dy)
{
    Vect2D vect;
    
    vect.dx = dx;
    vect.dy = dy;
    return vect;
}

Vect2D PHBoundVect2D(NSPoint start, Vect2D v, NSRect rc);

static inline NSPoint PHMovePointByVect(NSPoint pt, Vect2D offset)
{
    return NSMakePoint(pt.x + offset.dx, pt.y + offset.dy);
}

typedef struct _Interval
{
    float min;
    float max;
} Interval;

NSString * PHStringFromInterval(Interval intvl);
NSString * PHStringFromRect(NSRect rc);
NSString * PHStringFromSize(NSSize s);

#endif // _INCLUDED_PHGEOMETRY_H
