#include "PHGeometry.h"
#include <AppKit/NSAffineTransform.h>
#include <math.h>

//#define MIN(x, y) (((x) < (y)) ? (x) : (y))
//#define MAX(x, y) (((x) < (y)) ? (y) : (x))
#define DISTANCE(x, y) ( ((x) < (y)) ? ((y) - (x)) : ((x) - (y)) )
#define SIGN(x) ((x) > 0 ? 1 : -1)
#define ROUND(x) ((int)( (x) + 0.5 ))

NSPoint PHMovePoint(NSPoint pt, float dx, float dy)
{
    pt.x += dx;
    pt.y += dy;
    return pt;
}

NSPoint PHIntegralPoint(NSPoint pt)
{
    pt.x = floorf(pt.x);
    pt.y = floorf(pt.y);
    return pt;
}

NSPoint PHMoveOrigin(NSPoint pt, NSPoint delta_origin)
{
    pt.x -= delta_origin.x;
    pt.y -= delta_origin.y;
    return pt;
}

NSRect PHRectFromPoints(NSPoint a, NSPoint b)
{
    return NSMakeRect(MIN(a.x, b.x),
                      MIN(a.y, b.y),
                      DISTANCE(a.x, b.x),
                      DISTANCE(a.y, b.y));
}

NSSize PHIntegerSize(NSSize size)
{
  return PHIntegerSizePreserveAR( size, 0 );
}

NSSize PHIntegerSizePreserveAR(NSSize size, float aspectRatio)
{
    // Preserve span and aspect ratio as close as possible
    if ( aspectRatio > 0 )
    {
      // Start with the bigger size because it's probably already
      // adjusted to the required value
      float span =
        ( size.width > size.height ) ? size.width : size.height;
      if ( aspectRatio < 1 )
      {
        // here aspectRatio = y/x
        size.width = ROUND(span);
        size.height = ROUND(span / aspectRatio);
      }
      else
      {
        size.height = ROUND(span);
        size.width = ROUND(span * aspectRatio);
      }
    }
    else
    {
      size.width = ROUND( size.width );
      size.height = ROUND( size.height );
    }
    return size;
}

NSRect PHIntegerRect(NSRect rc)
{
  return PHIntegerRectPreserveAR( rc, 0 );
}

NSRect PHIntegerRectPreserveAR(NSRect rc, float aspectRatio)
{
    rc.origin = PHIntegralPoint( rc.origin );
    rc.size = PHIntegerSizePreserveAR( rc.size, 0 );
    return rc;
}

NSRect PHNearestIntegerRect(NSRect rc)
{
    // Round ll and ur points, then build rect from them
    NSPoint a = rc.origin;
    NSPoint b = NSMakePoint( NSMaxX(rc), NSMaxY(rc) );
    a.x = ROUND(a.x);
    a.y = ROUND(a.y);
    b.x = ROUND(b.x);
    b.y = ROUND(b.y);
    return PHRectFromPoints(a, b);
}

float PHDistance(NSPoint a, NSPoint b)
{
    //return MAX( DISTANCE(a.x, b.x), DISTANCE(a.y, b.y) );    
    return MIN( DISTANCE(a.x, b.x), DISTANCE(a.y, b.y) );    
}

NSPoint PHClosestPointInRect(NSPoint pt, NSRect rc)
{
    float x, y;
    if (pt.x < NSMinX(rc))
        x = NSMinX(rc);
    else if (pt.x > NSMaxX(rc))
        x = NSMaxX(rc);
    else
        x = pt.x;

    if (pt.y < NSMinY(rc))
        y = NSMinY(rc);
    else if (pt.y > NSMaxY(rc))
        y = NSMaxY(rc);
    else
        y = pt.y;

    return NSMakePoint(x, y);
}

NSPoint PHClosestPointOnRectEdge(NSPoint pt, NSRect rc)
{
    if (NSPointInRect(pt, rc) == NO)
        return PHClosestPointInRect(pt, rc);

    // find closest edge
    
    float d[4];
    d[0] = pt.x - NSMinX(rc);
    d[1] = pt.y - NSMinY(rc);
    d[2] = NSMaxX(rc) - pt.x;
    d[3] = NSMaxY(rc) - pt.y;

    // find minimum in the array d[]
    int i_min = 0;
    float d_min = d[i_min];
    int i;
    for (i=1; i < 4; ++i)
        if (d[i] < d_min)
            d_min = d[ i_min = i ];

    NSPoint result;
    switch (i_min)
    {
        case 0: result = NSMakePoint(NSMinX(rc), pt.y); break;
        case 1: result = NSMakePoint(pt.x, NSMinY(rc)); break;
        case 2: result = NSMakePoint(NSMaxX(rc), pt.y); break;
        case 3: result = NSMakePoint(pt.x, NSMaxY(rc)); break;
    }
    return result;
}

NSRect PHStretchRectToSpan( NSRect rc, int requiredSpan, float requiredAR,
                            NSRect scene )
{
    //NSLog( @"requiredSpan=%d  requiredAR=%f", requiredSpan, requiredAR);
    //NSLog( @"scene=%@", NSStringFromRect( scene ) );

    NSRect result = NSZeroRect;
    
    // here aspectRatio = y/x
    float ar = (requiredAR > 0) ? requiredAR : (rc.size.height/rc.size.width);

    NSSize newSize;

    newSize.width = (ar < 1) ? requiredSpan : requiredSpan / ar;
    newSize.height = (ar < 1) ? requiredSpan * ar : requiredSpan;

    //NSLog(@"newSize=%@", NSStringFromSize( newSize ) );
    
    // adjust size only if the new size fits into scene
    if ( newSize.width <= NSWidth( scene ) &&
         newSize.height <= NSHeight( scene ) )
    {
        float x_mid = NSMidX( rc );
        float y_mid = NSMidY( rc );

        result.origin = NSMakePoint( x_mid - newSize.width/2,
                                     y_mid - newSize.height/2 );
        result.size = newSize;
        result = PHIntegerRect( result );
        result = PHPushRectInsideBoundaries(result, scene);
    }

    //NSLog(@"result=%@", NSStringFromRect( result ) );
    return result;
}

NSRect PHPushRectInsideBoundaries(NSRect aRect, NSRect bound)
{
    NSRect result = aRect;
    if (aRect.size.width <= bound.size.width &&
        aRect.size.height <= bound.size.height)
    {
        if (result.origin.x < bound.origin.x)
            result.origin.x = bound.origin.x;
        else if (result.origin.x + result.size.width >
                 bound.origin.x + bound.size.width)
            result.origin.x =
                bound.origin.x + bound.size.width - result.size.width;

        if (result.origin.y < bound.origin.y)
            result.origin.y = bound.origin.y;
        else if (result.origin.y + result.size.height >
                 bound.origin.x + bound.size.height)
            result.origin.y =
                bound.origin.y + bound.size.height - result.size.height;
    }
    return result;
}

NSRect PHJMoveRectOrigin( NSRect rc, NSSize gridSize, NSSize imgSize )
{
    float dx = gridSize.width;
    float dy = gridSize.height;

    float x = NSMinX(rc);
    float y = imgSize.height - NSMaxY(rc);

    // Move rc.origin.x and rc.origin.y right and up.
    // We need this to counteract PHIntegerRect that moves the origin
    // in the opposite direction. See comment in Selection.m
    
    x = ceil(x / dx) * dx;

    // Make sure the rc is still within the image.
    if ( x + NSWidth(rc) > imgSize.width )
        x = floor((imgSize.width - NSWidth(rc)) / dx) * dx;

    y = floor(y / dy) * dy;

    rc.origin.x = x;
    rc.origin.y = imgSize.height - y - NSHeight(rc);

    return rc;
}

// Inflate bigger size to grid, set smaller size as close to AR as possible
NSRect PHJMoveRectKeepAR( NSRect rc, NSSize gridSize, NSSize imgSize )
{
    if ( NSIsEmptyRect(rc) )
        return NSZeroRect;
    
    float dx = gridSize.width;
    float dy = gridSize.height;

    float ar = rc.size.height / rc.size.width;

    float span, span_limit, small_limit, d_span, d_small;
    if ( ar > 1 )
    {
        span = rc.size.height;
        span_limit = imgSize.height;
        small_limit = imgSize.width;
        d_span = dy;
        d_small = dx;
    }
    else
    {
        span = rc.size.width;
        span_limit = imgSize.width;
        small_limit = imgSize.height;
        d_span = dx;
        d_small = dy;
    }

    span = ceil(span / d_span) * d_span;
    if ( span > span_limit )
        span = floor( span_limit / d_span ) * d_span;

    float small = (ar > 1) ? (span / ar) : (span * ar);
    small = rint(small / d_small) * d_small;

    if ( small > small_limit )
         small = floor( small_limit / d_small ) * d_small;

    rc.size.width = (ar > 1) ? small : span;
    rc.size.height = (ar > 1) ? span : small;

    return PHJMoveRectOrigin( rc, gridSize, imgSize );
}

// Keep bigger and inflate smaller side
NSRect PHJMoveRectKeepSpan( NSRect rc, NSSize gridSize, NSSize imgSize )
{
    if ( NSIsEmptyRect(rc) )
        return NSZeroRect;
    
    float dx = gridSize.width;
    float dy = gridSize.height;

    float ar = rc.size.height / rc.size.width;
    float small, small_limit, dd;
    if ( ar > 1 )
    {
        small = rc.size.width;
        small_limit = imgSize.width;
        dd = dx;
    }
    else
    {
        small = rc.size.height;
        small_limit = imgSize.height;
        dd = dy;
    }
    
    small = ceil(small / dd) * dd;
    if ( small > small_limit )
        small = floor(small_limit / dd) * dd;

    if ( ar > 1 )
        rc.size.width = small;
    else
        rc.size.height = small;
    
    return PHJMoveRectOrigin( rc, gridSize, imgSize );
}

NSSize PHRotateSize(NSSize val, int degrees)
{
    // TV: need conversion degrees -> enum
    if ( degrees == 90 || degrees == 270 )
        return NSMakeSize( val.height, val.width );
    else
        return val;
}

NSSize PHScaledSize( NSSize src, int newSpan, float * resultScale )
{
    NSSize scaled_size = src;
    double scale = 1.0;

    int srcLargeSide = MAX( src.width, src.height );
    int srcSmallSide = MIN( src.width, src.height );

    if ( 0 < newSpan && newSpan < srcLargeSide )
    {
        scale = (double)newSpan / srcLargeSide;
        int small_side = floor( scale * srcSmallSide );
        scaled_size.width = newSpan;   // assume the large size is width
        scaled_size.height = small_side;
        if ( src.width < src.height )
            scaled_size = PHRotateSize( scaled_size, 90 ); // we assumed wrond
    }
    if ( resultScale )
        *resultScale = (float)scale;

    return scaled_size;
}

float PHMaxSpan(NSSize size)
{
    return MAX( size.width, size.height );
}

NSAffineTransform * PHReverseTransform(NSAffineTransform * xform)
{
    NSAffineTransform * reverse = [xform copy];
    [reverse invert];

    return AUTORELEASE(reverse);
}

Vect2D PHBoundVect2D(NSPoint start, Vect2D v, NSRect rc)
{
    // Check that start point is in rect
    
    // Move referrence frame origin to the origin of rc
    //start.x -= rc.origin.x;
    //start.y -= rc.origin.y;

    float shape = fabs(v.dy / v.dx);
    float dx1 = MIN(fabsf(v.dx),
                    (v.dx > 0) ? rc.size.width - start.x + rc.origin.x :
                    start.x - rc.origin.x);

    float dy1 = dx1 * shape;


    float dy2 = MIN(fabsf(v.dy),
                    (v.dy > 0) ? rc.size.height - start.y + rc.origin.y :
                    start.y - rc.origin.y);

    float dx2 = (shape != 0) ? dy2 / shape : 0;

    // take minimal size

    float dx = SIGN(v.dx) * MIN(dx1, dx2);
    float dy = SIGN(v.dy) * MIN(dy1, dy2);

    return PHMakeVect2D(dx, dy);
}

NSString * PHStringFromInterval(Interval intvl)
{
    return [NSString stringWithFormat:
                         @"{min=%g max=%g}", intvl.min, intvl.max];
}

NSString * PHStringFromRect(NSRect rc)
{
    return [NSString stringWithFormat: @"{%gx%g%+g%+g}",
                     NSWidth(rc), NSHeight(rc), rc.origin.x, rc.origin.y ];
}

NSString * PHStringFromSize(NSSize s)
{
    return [NSString stringWithFormat: @"{%gx%g}", s.width, s.height];
}
