#ifndef _INCLUDED_Selection_H
#define _INCLUDED_Selection_H

#include <Foundation/NSObject.h>
#include <Foundation/NSGeometry.h>
#include "PictureView.h"

@class NSAffineTransform;
@class NSEvent;
@class NSImage;
@class NSCursor;
@class MouseTracker;

@interface Selection : NSObject
{
    PictureView * _view;

    NSRect  _selectedInImage;   // In image coordinates

    NSRect _selection;          // In the sketch coordinates
    NSRect _previousSelection;  // In the sketch coordinates

    MouseTracker * _tracker;
}


- (id) initWithView: (PictureView *) view;
- (id) initWithView: (PictureView *) view selectedInImage: (NSRect) rc;

- (void) update;
- (void) adjustToJPEGGrid: (NSSize) blockSize;

// 'pt' is in sketch coordinates
- (void) mouseDown: (NSPoint) pt;
- (void) mouseDragged: (NSPoint) pt;
- (void) mouseUp: (NSPoint) pt;

// Returns an array of CursorRectInfo's
- (NSArray *) cursorRects;

- (NSCursor *) trackingCursor;

// Returns YES if selection is non-empty
- (BOOL) valid;

// Image coordinates
- (NSRect) selectedInImage;

// Sketch coordinates
- (NSRect) selectedInSketch;

// Sketch coordinates
- (NSRect) updatedInSketch;

// Returns YES if the point is outside of the outer boundary of the sel. egde
- (BOOL) isOutOfSelPoint: (NSPoint) pt;

- (NSImage *) mask;

// 'rect' is in sketch coordinates
- (NSImage *) maskForRect: (NSRect) rect;


@end // interface Selection

#endif // _INCLUDED_Selection_H
