#ifndef _INCLUDED_PIXELOP_H_
#define _INCLUDED_PIXELOP_H_

#if defined(__sun__)

// Solaris
#include <sys/isa_defs.h>

#  if defined(BIG_ENDIAN)
#    define __BYTE_ORDER __BIG_ENDIAN
#  else
#    define __BYTE_ORDER __LITTLE_ENDIAN
#  endif

#else

// Linux
#include <endian.h>

#endif // ! defined(__sun__)

#if __BYTE_ORDER == __LITTLE_ENDIAN
#define SWAP_BYTES(x) ((x) = ( (x)<<8 | (x)>>8 ))
#else
#define SWAP_BYTES(x)
#endif // __BYTE_ORDER == __LITTLE_ENDIAN

#define COMPONENT_VALUE( pix, c, bitsPerSample, value )     \
{                                                           \
    unsigned char * pixel = (pix);                          \
    switch ( bitsPerSample )                                \
    {                                                       \
        case 4:                                             \
        {                                                   \
            unsigned char ch = pixel[ c/2 ];                \
            ch >>= ((c+1)%2) * 4;                           \
            ch &= 0x0f;                                     \
            value = ch;                                     \
            break;                                          \
        }                                                   \
        case 8:                                             \
        {                                                   \
            value = pixel[c];                               \
            break;                                          \
        }                                                   \
        case 12:                                            \
        {                                                   \
            unsigned short sh = *(unsigned short *)(pixel + 3*c/2); \
            SWAP_BYTES(sh);                                 \
            sh >>= ((c+1)%2) * 4;                           \
            value = sh & 0x0fff;                            \
            break;                                          \
        }                                                   \
        case 16:                                            \
        {                                                   \
            unsigned short sh = *(unsigned short *)(pixel + 2*c);   \
            SWAP_BYTES(sh);                                 \
            value = sh;                                     \
            break;                                          \
        }                                                   \
        default:                                            \
        {                                                   \
            value = 0;                                      \
            break;                                          \
        }                                                   \
    }                                                       \
}

#define SET_COMPONENT_VALUE( pixel, c, bitsPerSample, value )       \
{                                                                   \
    switch ( bitsPerSample )                                        \
    {                                                               \
        case 4:                                                     \
        {                                                           \
            unsigned char ch = (unsigned char)(value) & 0x0f;       \
            ch <<= ((c+1)%2) * 4;                                   \
            unsigned char mask = 0xf0 >> ((c+1)%2) * 4;             \
            *(pixel + c/2) &= mask;                                 \
            *(pixel + c/2) |= ch;                                   \
            break;                                                  \
        }                                                           \
        case 8:                                                     \
        {                                                           \
            *(pixel + c) = (unsigned char)(value);                  \
            break;                                                  \
        }                                                           \
        case 12:                                                    \
        {                                                           \
            unsigned short sh = (unsigned short)(value) & 0x0fff;   \
            sh <<= ((c+1)%2) * 4;                                   \
            unsigned short mask = 0xf000 >> ((c+1)%2) * 12;         \
            SWAP_BYTES(sh);                                         \
            SWAP_BYTES(mask);                                       \
            unsigned short * p = (unsigned short *)(pixel + 3*c/2); \
            *p &= mask;                                             \
            *p |= sh;                                               \
            break;                                                  \
        }                                                           \
        case 16:                                                    \
        {                                                           \
            unsigned short sh = (unsigned short)(value);            \
            SWAP_BYTES(sh);                                         \
            *(unsigned short *)(pixel + 2*c) = sh;                  \
            break;                                                  \
        }                                                           \
        default:                                                    \
        {                                                           \
            break;                                                  \
        }                                                           \
    }                                                               \
}

#endif // _INCLUDED_PIXELOP_H_
