#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdio.h>
#include <fcntl.h>
#include <errno.h>

int copy( const char * oldpath, const char * newpath);
ssize_t writeall(int fd, const void * buf, size_t count);

// Like system call rename(), but copies and unlinks the oldpath
// if rename() fails with EXDEV error code (not on same filesystem)

int renameFile( const char * oldpath, const char * newpath )
{
    static char buffer[256];

    // Try rename() first
    
    errno = 0;
    int renameResult = rename( oldpath, newpath );
    if ( renameResult == 0 )
        return 0;

    if ( renameResult == -1 && errno != EXDEV )
    {
        printf("[1]\n");
        sprintf( buffer, "rename(%s, %s) failed", oldpath, newpath );
        perror( buffer );
        return -1;
    }

    // Copy
    errno = 0;
    if ( copy( oldpath, newpath ) == -1 )
        return -1;

    if ( unlink(oldpath) == -1 )
    {
        sprintf( buffer, "unlink(%s) failed", oldpath );
        perror(buffer);
        return -1;
    }

    return 0; // success
}

int copy( const char * oldpath, const char * newpath)
{
    static char buffer[256];
    static char block[1024];

    int fdin = open( oldpath, O_RDONLY );
    if ( fdin == -1 )
    {
        sprintf( buffer, "open(%s) failed", oldpath );
        perror( buffer );
        return -1;
    }

    int fdout = open( newpath, O_WRONLY | O_CREAT | O_TRUNC, 0666 );
    if ( fdout == -1 )
    {
        sprintf( buffer, "open(%s) failed", newpath );
        perror( buffer );

        close(fdin);
        return -1;
    }

    ssize_t nread;
    ssize_t nwritten;
    while ( ( nread = read(fdin, block, sizeof(block) ) ) > 0 )
    {
        nwritten = writeall( fdout, block, nread );
        if ( nwritten != nread )
        {
            sprintf( buffer, "write to file %s failed", newpath );
            perror( buffer );
            goto cleanup;
        }
    }
    if ( nread == -1 )
    {
        sprintf( buffer, "read from file %s failed", oldpath );
        perror( buffer );
        goto cleanup;
    }

    close(fdin);
    close(fdout);

    errno = 0;
    return 0; // success

 cleanup:
    close(fdin);
    close(fdout);
    return -1; 
}

ssize_t writeall(int fd, const void * buf, size_t count)
{
    ssize_t nwritten = 0;
    while ( count )
    {
        ssize_t n = write( fd, buf + nwritten, count );
        if ( n == -1 )
            return -1;
        nwritten += n;
        count -= n;
    }
    return nwritten;
}
